import copy

from pathlib import Path

from damask import __version__
from damask_parse.writers import write_geom as write_geom_
from damask_parse.writers import write_load_case
from damask_parse.writers import write_material as write_material_
from damask_parse import write_numerics as write_numerics_


def write_input_files(
    path,
    volume_element,
    load_case,
    damask_solver,
    homogenization,
    damask_phases,
    single_crystal_parameters,
    damask_numerics,
):
    geom_path = path
    write_geom(geom_path, volume_element)
    write_load(load_case, damask_solver)
    write_material(
        volume_element,
        homogenization,
        damask_phases,
        single_crystal_parameters,
    )
    if damask_numerics:
        write_numerics(damask_numerics)


def write_geom(path, volume_element):
    write_geom_(dir_path=path.parent, volume_element=volume_element, name=path.name)


def write_load(load_case, damask_solver):
    path = Path("load.yaml")
    load_steps = []
    for step in load_case.steps:
        dct = step.to_dict()
        dct["def_grad_aim"] = dct.pop("target_def_grad", None)
        dct["def_grad_rate"] = dct.pop("target_def_grad_rate", None)
        load_steps.append(dct)

    write_load_case(
        dir_path=path.parent,
        load_cases=load_steps,
        solver=damask_solver,
        name=path.name,
        write_2D_arrs=(__version__ != "3.0.0-alpha3"),
    )


def write_material(
    volume_element,
    homogenization,
    damask_phases,
    single_crystal_parameters,
):
    path = Path("material.yaml")
    if single_crystal_parameters is not None:
        # merge single-crystal properties into phases:
        damask_phases = copy.deepcopy(damask_phases)
        for phase_label in damask_phases.keys():
            SC_params_name = damask_phases[phase_label]["mechanical"]["plastic"].pop(
                "single_crystal_parameters", None
            )
            if SC_params_name:
                SC_params = single_crystal_parameters[SC_params_name]
                damask_phases[phase_label]["mechanical"]["plastic"].update(**SC_params)

    write_material_(
        dir_path=path.parent,
        homog_schemes=homogenization,
        phases=damask_phases,
        volume_element=volume_element,
        name=path.name,
    )


def write_numerics(damask_numerics):
    path = Path("numerics.yaml")
    write_numerics_(dir_path=path.parent, numerics=damask_numerics, name=path.name)



from damask_parse.utils import volume_element_from_2D_microstructure


def generate_volume_element_extrusion(
    microstructure_image,
    depth,
    image_axes,
    homog_label,
    phase_label,
    phase_label_mapping,
):
    volume_element = volume_element_from_2D_microstructure(
        microstructure_image=microstructure_image,
        homog_label=homog_label,
        phase_label=phase_label,
        phase_label_mapping=phase_label_mapping,
        depth=depth,
        image_axes=image_axes,
    )
    volume_element["grid_size"] = tuple(int(i) for i in volume_element["grid_size"])
    volume_element["size"] = tuple(float(i) for i in volume_element["size"])
    return {"volume_element": volume_element}



import pytest
import matflow as mf


def pytest_addoption(parser):
    parser.addoption(
        "--integration",
        action="store_true",
        default=False,
        help="run integration-like workflow submission tests",
    )


def pytest_configure(config):
    config.addinivalue_line(
        "markers",
        "integration: mark test as an integration-like workflow submission test to run",
    )
    mf.run_time_info.in_pytest = True


def pytest_collection_modifyitems(config, items):
    if config.getoption("--integration"):
        # --integration in CLI: only run these tests
        for item in items:
            if "integration" not in item.keywords:
                item.add_marker(
                    pytest.mark.skip(reason="remove --integration option to run")
                )
    else:
        for item in items:
            if "integration" in item.keywords:
                item.add_marker(
                    pytest.mark.skip(reason="add --integration option to run")
                )


def pytest_unconfigure(config):
    mf.run_time_info.in_pytest = False


@pytest.fixture
def null_config(tmp_path):
    if not mf.is_config_loaded:
        mf.load_config(config_dir=tmp_path)
    mf.run_time_info.in_pytest = True


@pytest.fixture
def new_null_config(tmp_path):
    mf.load_config(config_dir=tmp_path)
    mf.load_template_components(warn=False)
    mf.run_time_info.in_pytest = True



from pathlib import Path
from damask_parse.readers import read_HDF5_file
from damask_parse.utils import generate_viz


def process_HDF5(damask_hdf5_file, damask_post_processing, VE_response_data, damask_viz):
    VE_response = read_HDF5_file(
        hdf5_path=damask_hdf5_file,
        operations=damask_post_processing,
        **VE_response_data,
    )
    generate_viz(hdf5_path=damask_hdf5_file, viz_spec=damask_viz, parsed_outs=VE_response)

    return VE_response
